import React, {
  forwardRef,
  useEffect,
  useReducer,
  useRef,
  useState,
} from "react";
import {
  Text,
  TouchableOpacity,
  StyleSheet,
  View,
  Image,
  Alert,
  Switch,
} from "react-native";
import { async } from "validate.js";
import Colors from "../common/Colors";
import { SCREEN_HEIGHT, SCREEN_WIDTH } from "../common/Constants";
import Database from "../../../../Components/common/Database";
import { PrimaryButton, TextInput } from "../common/ThemedComponents";
import Orientation from "react-native-orientation";
import AnimationEaseLinear from "./AnimationEaseLinear";
import ViewShot, { captureRef } from "react-native-view-shot";
import UploadFile from "react-native-digitcamera/src/Screens/UploadFile";
import Modal from "../common/Modal";
import InstructionsScreen from "./InstructionsScreen";
import Icon from "react-native-vector-icons/dist/FontAwesome";
import MIcon from "react-native-vector-icons/MaterialIcons";
import Steps from "react-native-digitcamera/src/Screens/Steps";
import style from "../../../../Components/common/CommonStyle";

export const zIndex_main_view = 100,
  zIndex_image_instruction = 125,
  zIndex_snapshot_preview_image = 150,
  zIndex_moving_thumbnail = 250,
  zIndex_bottom_instruction = 300,
  zIndex_stop_recording = 325,
  zIndex_start_recording = 400,
  zIndex_flash_torch = 124,
  zIndex_flash_torchVDO = 326;
zIndex_steps_instruction = 312;

const RCB1 = "RCB1",
  RCB2 = "RCB2",
  DBD = "DBD",
  VDO = "VDO";

export const NO_OF_POSES = 12; // 1 is width of thumbnail itself
export const OBJ_WIDTH = SCREEN_HEIGHT / NO_OF_POSES;
const newPosePeriod = SCREEN_HEIGHT / NO_OF_POSES;
export const ANIMATION_TRACK = SCREEN_HEIGHT;
export const ANIM_DURATION = 2 * 60 * 1000;
const EXTRA_TIME = 1 * 60 * 1000;
export const DURATION_PERIOD = ANIM_DURATION / 19; // 19 is the no. of instructions coming from back end
const PAUSE_PART_NAME = "doors are closed";

var locationRetryCount = 0;
let realm = Database.getRealm();
var instructionInDb = realm.objects("PIVechicleInstruction");
var pause = false;
var animValue = 0;
export const StencilAnimation = forwardRef((props, cameraRef) => {
  // const [pause, setPause] = useState(false)
  const [easeAnimation, setEaseAnimation] = useState(false);
  const [startRecording, setStartRecording] = useState(false);
  const [disableStopVideoButton, setDisableStopVideoButton] = useState(true);
  var [currentLatitude, setCurrentLatitude] = useState("");
  var [currentLongitude, setCurrentLongitude] = useState("");
  const [imagePreview, setImagePreview] = useState(false);
  const [imagePreviewUri, setImagePreviewUri] = useState("");
  const [dateTime, setDateTime] = useState("");
  const [odometerReadings, setOdometerReadings] = useState("");
  const [instructionsModalVisible, setInstructionsModalVisible] =
    useState(false);
  const viewShotRef = useRef();
  // const [enableRCBPreview, setEnableRCBPreview] = useState(false);

  var dbInstr = instructionInDb[0]?.instuction;
  // if (__DEV__ && (dbInstr === undefined || dbInstr.length <= 0)) {
  //   console.log("**** dbInstruction not available taking hard coded");
  //dbInstr = JSON.parse('[{\"surveyStepsId\":1053,\"createdDate\":\"Feb21,202012:15:05PM\",\"isLast\":false,\"lastModifiedDate\":\"Feb21,202012:15:05PM\",\"stepAssests\":\"https://cdn.godigit.com/digitEdgeAssets/Motor/4W_PI_RCB1_V2.jpg\",\"stepStencilAssests\":\"\",\"stepTitle\":\"RCBook/Card\",\"stepTitleHn\":\"आरसीबुक/कार्ड\",\"imageName\":\"RCB1\",\"displayOrder\":160,\"type\":\"image\",\"TSurveyInstructions\":[{\"surveyInstructionsId\":1740,\"createdDate\":\"May18,202212:25:18PM\",\"instructionData\":\"TakeapictureofthefrontofyourRCbook/card.\",\"lastModifiedDate\":\"May18,202212:25:18PM\",\"instructionTime\":1,\"instructionDataHn\":\"अपनीआरसीबुक/कार्डकेसामनेकीतस्वीरलें।\",\"effectiveDate\":\"May18,202212:27:19PM\",\"displayorder\":\"1\",\"type\":\"\"}],\"effectiveDate\":\"Dec17,202203:18:48PM\"},{\"surveyStepsId\":1056,\"createdDate\":\"Feb21,202012:15:05PM\",\"isLast\":false,\"lastModifiedDate\":\"Feb21,202012:15:05PM\",\"stepAssests\":\"https://cdn.godigit.com/digitEdgeAssets/Motor/4W_PI_RCB2_V2.jpg\",\"stepStencilAssests\":\"\",\"stepTitle\":\"RCBook/Card\",\"stepTitleHn\":\"आरसीबुक/कार्ड\",\"imageName\":\"RCB2\",\"displayOrder\":180,\"type\":\"image\",\"TSurveyInstructions\":[{\"surveyInstructionsId\":1741,\"createdDate\":\"May18,202212:25:18PM\",\"instructionData\":\"TakeapictureofthebackofyourRCbook/card.\",\"lastModifiedDate\":\"May18,202212:25:18PM\",\"instructionTime\":2,\"instructionDataHn\":\"अपनीआरसीबुक/कार्डकेपीछेकीतस्वीरलें।\",\"effectiveDate\":\"May18,202212:27:19PM\",\"displayorder\":\"2\",\"type\":\"\"}],\"effectiveDate\":\"Dec17,202203:18:48PM\"},{\"surveyStepsId\":1054,\"createdDate\":\"Feb21,202012:15:05PM\",\"description\":\"Makesuretheengineisonwhiletakingtheodometerkmreading\",\"isLast\":false,\"descriptionHn\":\"सुनिश्चितकरेंकिओडोमीटरकिमीरीडिंगलेतेसमयइंजनचालूहै\",\"lastModifiedDate\":\"Feb21,202012:15:05PM\",\"stepAssests\":\"https://cdn.godigit.com/digitEdgeAssets/Motor/MZBG2T7_Odometer-01_V2.jpg\",\"stepStencilAssests\":\"\",\"stepTitle\":\"Pleaseentertheodometerkmreading\",\"stepTitleHn\":\"कृपयाओडोमीटरकिमीरीडिंगदर्जकरें\",\"imageName\":\"DBD\",\"displayOrder\":205,\"type\":\"input\",\"TSurveyInstructions\":[],\"effectiveDate\":\"Dec17,202203:18:48PM\"},{\"surveyStepsId\":1055,\"createdDate\":\"Feb21,202012:15:05PM\",\"description\":\"Shootvideofromadistancetocapturetheentirevehicle\",\"isLast\":false,\"descriptionHn\":\"पूरीगाड़ीकोफ़ोकसकरनेकेलिएदूरसेवीडियोशूटकरें\",\"lastModifiedDate\":\"Feb21,202012:15:05PM\",\"stepAssests\":\"https://d2h44aw7l5xdvz.cloudfront.net/videos/self-inspection-for-car.mp4\",\"stepStencilAssests\":\"\",\"stepTitle\":\"Takea360°video\",\"stepTitleHn\":\"360°वीडियोलें\\\"\",\"imageName\":\"VDO\",\"displayOrder\":220,\"type\":\"video\",\"TSurveyInstructions\":[{\"surveyInstructionsId\":1742,\"createdDate\":\"May18,202212:30:48PM\",\"instructionData\":\"Wellrecordinsidetheenginebaysomakesurethebonnetiskeptopen.\",\"lastModifiedDate\":\"May18,202212:30:48PM\",\"instructionTime\":3,\"instructionDataHn\":\"हमइंजनबेकेअंदररिकॉर्डकरेंगेइसलिएसुनिश्चितकरेंकिबोनटखुलारखागयाहै\",\"effectiveDate\":\"May18,202212:32:26PM\",\"instructionTitle\":\"Pleaseopenthebonnetofthecar\",\"okButtonText\":\"Ihaveopenedthebonnet\",\"instructionAssests\":\"https://d2h44aw7l5xdvz.cloudfront.net/CRM/PRM/open_bonnet.png\",\"displayorder\":\"3\",\"type\":\"image\"},{\"surveyInstructionsId\":1743,\"createdDate\":\"May18,202212:30:48PM\",\"instructionData\":\"TheengineshouldbeONthroughouttheinspection.\",\"lastModifiedDate\":\"May18,202212:30:48PM\",\"instructionTime\":3,\"instructionDataHn\":\"पूरेनिरीक्षणकेदौरानइंजनचालूरहनाचाहिए\",\"effectiveDate\":\"May18,202212:32:26PM\",\"instructionTitle\":\"Startyourvehicle\",\"okButtonText\":\"Ihavestartedthevehicle\",\"instructionAssests\":\"https://d2h44aw7l5xdvz.cloudfront.net/CRM/PRM/key_start.png\",\"displayorder\":\"4\",\"type\":\"image\"},{\"surveyInstructionsId\":1744,\"createdDate\":\"May18,202212:30:48PM\",\"instructionData\":\"Takesometime&locateyourchassisnumber\",\"lastModifiedDate\":\"May18,202212:30:48PM\",\"instructionTime\":3,\"instructionDataHn\":\"कुछसमयनिकालेंऔरअपनाचेसिसनंबरखोजें\",\"effectiveDate\":\"May18,202212:32:26PM\",\"instructionTitle\":\"Findyourchassisnumber\",\"okButtonText\":\"Ihavefoundthechassisno.\",\"instructionAssests\":\"https://d2h44aw7l5xdvz.cloudfront.net/CRM/PRM/chassis_find.png\",\"displayorder\":\"5\",\"type\":\"image\"},{\"surveyInstructionsId\":1745,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Whatisembossedchassisno.?\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"Whatisembossedchassisno.?\",\"displayorder\":\"6\",\"type\":\"image\"},{\"surveyInstructionsId\":1746,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Whatisembossedchassisno.?\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"Whatisembossedchassisno.?\",\"displayorder\":\"7\",\"type\":\"image\"},{\"surveyInstructionsId\":1747,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Makesurethechassisnumberisvisibleonthescreen\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"8\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1748,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Makesureallthedoorsareclosed.\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"9\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1749,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Facethefrontofthecar\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"10\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1750,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Letslookinsidetheenginebay\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"11\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1751,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Facethefrontofthecar\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"12\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1752,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Numberplate&frontwinshieldshouldbevisible\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"13\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1753,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Movetowardsthefrontpassengerside\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"14\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1754,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Ensurethefullcarisintheframe\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"15\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1755,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Completesideofthecarshouldbeseen\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"16\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1756,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Stepbackifyouneedtofitthecarintheframe\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"17\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1757,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Movetothebackofthecar\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"18\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1758,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Numberplate&backwindshieldshouldbevisible\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"19\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1759,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Movetothedriversside\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"20\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1760,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Completesideofthecarshouldbevisible\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"21\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1761,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Movetothefrontofthecar\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"22\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1762,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Moveclosertothewindshield\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"23\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1763,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Pleasesitonthedriversseat\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"24\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1764,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Allcornersofthewindshieldshouldbefilmedfrominside\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"25\",\"type\":\"instruction\"},{\"surveyInstructionsId\":1765,\"createdDate\":\"May31,202202:40:44PM\",\"instructionData\":\"Letstakealookattheodometer\",\"lastModifiedDate\":\"May31,202202:40:44PM\",\"instructionTime\":2,\"effectiveDate\":\"May31,202202:40:44PM\",\"instructionTitle\":\"\",\"displayorder\":\"26\",\"type\":\"instruction\"}],\"effectiveDate\":\"Dec17,202203:18:48PM\"}]');
  // } else {
  dbInstr = JSON.parse(dbInstr);
  // }
  var [instructionArr, setInstructionArr] = useState(dbInstr);
  const originalState = instructionArr[0];
  const [instructionManager, moveToNextStencil] = useReducer(
    reducer,
    originalState
  );
  /*torch mode*/
  const [torchState, setTorchState] = useState(false);
  var posePeriodInterval = useRef();
  var animObjRef = useRef();
  var animatedRef = useRef();
  var timeoutRef = useRef();
  var resumeTimerRef = useRef();
  var pauseTimerRef = useRef();
  var extraTimeRef = useRef();
  var nextStepRef = useRef();
  var prevStepRef = useRef();
  var resetStepRef = useRef();

  function reducer(state = originalState, action) {
    let newState;
    switch (action.type) {
      case "init":
        console.log("***** reducer init: ");
        newState = instructionArr[0];
        break;
      case "next":
        const oldIndex = instructionArr.findIndex(
          (obj) => obj.displayOrder === state.displayOrder
        );
        const nextIndex = oldIndex + 1;
        console.log("***** reducer next " + nextIndex);
        const newInstruction = instructionArr[nextIndex];
        newState = newInstruction;
        break;
      case "prev":
        const oldPrevIndex = instructionArr.findIndex(
          (obj) => obj.displayOrder === state.displayOrder
        );
        const nextPrevIndex = oldPrevIndex - 1;
        console.log("***** reducer prev " + nextPrevIndex);
        newState = instructionArr[nextPrevIndex];
        break;
      case "toImageName" /* @NEWORDER */:
        const newInstructions = instructionArr.find(
          (item) => item.imageName === action.payload
        );
        newState = newInstructions;
        break;
    }
    console.log("**** reducer returns: " + newState.imageName);
    return newState;
  }
  useEffect(() => {
    getLatLong();
    getTime();
  }, [instructionManager]);

  /*@NEWORDER*/
  useEffect(() => {
    if (props.toVDO !== undefined && props.toVDO) {
      setTimeout(() => {
        toImageName("VDO");
      }, 1);
    }
  }, []);

  useEffect(() => {
    Orientation.lockToLandscapeLeft();

    const myFun = async () => {
      await getInstructionsFromDb();
    };
    myFun();
    return () => {};
  }, []);

  const animationListener = (anim, value) => {
    if (animObjRef.current === undefined) {
      animObjRef.current = anim;
    }
    animValue = value;
    console.log(
      "*** animationListener anim:  duration: " +
        ANIM_DURATION +
        " value: " +
        animValue +
        " toValue: " +
        ANIMATION_TRACK
    );
  };

  const posePeriodIntervalFun = () => {
    posePeriodInterval.current = setInterval(() => {
      // nextStencil();
      if (nextStepRef.current && !pause) {
        const stepManager = nextStepRef.current();
        if (
          stepManager !== undefined &&
          stepManager?.instructionData.indexOf(PAUSE_PART_NAME) >= 0
        ) {
          // setPause(true)
          pause = true;
          if (animatedRef.current != undefined) {
            animatedRef.current[0].stop();
          }
          if (pauseTimerRef.current != undefined) {
            pauseTimerRef.current();
          }
          Alert.alert(
            "Close the bonnet",
            "If bonnet is not closed, please close the bonnet",
            [
              {
                text: "Continue",
                onPress: () => {
                  // setPause(false)
                  if (resumeTimerRef.current != undefined) {
                    resumeTimerRef.current();
                  }
                  pause = false;
                  console.log("*** StencilAnimation waiting for chassis");
                  animObjRef.current.setValue(animValue);
                  if (animatedRef.current != undefined) {
                    animatedRef.current[0].start();
                  }
                },
              },
            ]
          );
        }
      }
    }, DURATION_PERIOD);
    return posePeriodInterval.current;
  };

  const getInstructionsFromDb = async () => {
    let dbInstruction = instructionInDb[0]?.instuction;
    if (dbInstruction === undefined || dbInstruction.length <= 0) {
      props.navigation.navigate("WithQuote");
      Alert.alert(
        "Try Again!",
        "We are unable to load guiding information for you."
      );
      return;
    }
    instructionArr = dbInstruction;
    console.log("**** getInstructionFromDb: " + instructionArr);
    nextStencil("init");
  };
  const nextStencil = (instructionType = "next") => {
    moveToNextStencil({ type: instructionType });
  };

  const prevStencil = (instructionType = "prev") => {
    moveToNextStencil({ type: instructionType });
  };

  /* @NEWORDER */
  const toImageName = (imageName) => {
    moveToNextStencil({ type: "toImageName", payload: imageName });
  };

  const imageInstructionTop = (instruction) => {
    return (
      <View style={styles.topInstructionView}>
        <Text style={styles.topInstructionText}>{instruction.stepTitle}</Text>
      </View>
    );
  };

  const instructionWithImage = (instruction) => {
    return (
      <View style={styles.instructionImageView}>
        {imageInstructionTop(instruction)}
        <Image
          key={instruction.displayOrder}
          style={styles.stencilStyle}
          resizeMode="stretch"
          source={{ uri: instruction.stepAssests }}
        />
        {true && (
          <TextInput
            returnKeyType="done"
            keyboardType={"number-pad"}
            style={{ flex: 0.2, marginHorizontal: 5, marginVertical: 5 }}
            label={"Enter Odometer Reading (in Km)"}
            value={"" + odometerReadings}
            onChangeText={(value) => {
              setOdometerReadings(value);
            }}
            errorMsge={""}
          />
        )}
        {imageInstructionBottom(instruction)}
      </View>
    );
  };

  const imageInstructionBottom = (instruction) => {
    return (
      <View style={styles.bottomInstructionView}>
        <Text style={styles.bottmInstructionText}>{instruction.stepTitle}</Text>
        <PrimaryButton
          style={styles.bottomInstructionButton}
          disabled={false}
          onPress={() => {
            getButtonAction(instruction)();
          }}
        >
          {getButtonName(instruction)}
        </PrimaryButton>
      </View>
    );
  };

  const onRetryCallback = (uri, DocumentImageName) => {
    if (
      DocumentImageName === RCB1 ||
      DocumentImageName === RCB2 ||
      DocumentImageName === DBD
    ) {
    } else if (DocumentImageName === VDO) {
    }
  };

  const onProceedCallback = (uri, DocumentImageName) => {
    if (
      DocumentImageName === RCB1 ||
      DocumentImageName === RCB2 ||
      DocumentImageName === DBD
    ) {
      console.log("**** ProceedCallback if UploadFile: " + DocumentImageName);
      UploadFile(uri, DocumentImageName, ".jpg", () => {
        props.navigation.navigate("WithQuote");
      });
      nextStencil("next");
    } else if (DocumentImageName === VDO) {
      // ***** IMPORTANT *******
      // this Logic is written in Camera.js
      // considering VDO is last file else we can check array[array.length-1] is last doc.
      console.log(
        "**** ProceedCallback else before UploadFile: " + DocumentImageName
      );
      props.navigation.navigate("VideoPIProgress");
      UploadFile(uri, DocumentImageName, ".jpg", () => {
        props.navigation.navigate("WithQuote");
      });
      console.log(
        "**** ProceedCallback after UploadFile: " + DocumentImageName
      );
      console.log(
        "**** ProceedCallback after navigate Progress: " + DocumentImageName
      );
    }
  };

  const goToPreviewMedia = (uri, documentImageName) => {
    props.navigation.navigate("PreviewMedia", {
      uri: uri,
      DocumentImageName: documentImageName,
      onRetry: onRetryCallback,
      onProceed: onProceedCallback,
    });
  };

  const getButtonAction = (instruction) => {
    if (instruction.imageName === RCB1 || instruction.imageName === RCB2) {
      const takeSnapshot = async () => {
        const options = {
          quality: 0.2,
          base64: true,
          orientation: "landscapeLeft",
        };
        console.log("*** takeSnapshot before takePictureAsync");
        const data = await props.myCamera.takePictureAsync(options);
        console.log("*** takeSnapshot after takePictureAsync: " + data.uri);
        setImagePreview(true);
        setImagePreviewUri(data.uri);
        // Alert.alert("Uri", data.uri);
        console.log("*** takeSnapshot before captureRef");
        const captureTimeout = setTimeout(() => {
          clearTimeout(captureTimeout);
          captureRef(viewShotRef, {
            format: "jpg",
            quality: 0.8,
          }).then(
            (uri) => {
              console.log("*** takeSnapshot in then captureRef: " + uri);
              setImagePreview(false);
              goToPreviewMedia(uri, instruction.imageName);
            },
            (error) => {
              console.log("*** takeSnapshot in error captureRef: " + error);
              // Alert.alert("ViewShot Uri Error", "" + error);
              setImagePreview(false);
            }
          );
        }, 700);
      };
      return takeSnapshot;
    } else if (instruction.imageName === VDO) {
      return stopScreenRecordingCallback;
    } else if (instruction.imageName === DBD) {
      const getODO = async () => {
        if (odometerReadings === undefined || odometerReadings.length <= 0) {
          Alert.alert("Enter valid readings");
          return;
        }
        savaDataToRealm(odometerReadings);
        nextStencil("next");
      };
      return getODO;
    }
    return () => {
      Alert.alert(
        "Oops!",
        "We are unable to take picture for " + instruction?.imageName
      );
    };
  };

  const savaDataToRealm = (value) => {
    console.log("**** saveDataToRealm Odometer: " + value);
    realm.write(() => {
      console.log("**** saveDataToRealm before Odometer: " + value);
      realm.delete(realm.objects("PIOdometerData"));
      realm.create("PIOdometerData", {
        readings: value,
      });
    });
    console.log(
      "**** saveDataToRealm after Odometer: " +
        JSON.stringify(realm.objects("PIOdometerData"))
    );
  };

  const getButtonName = (instruction) => {
    if (instruction.imageName === RCB1 || instruction.imageName === RCB2) {
      return "Capture";
    } else if (instruction.imageName === VDO) {
      return "Stop";
    } else if (instruction.imageName === DBD) {
      return "Submit";
    }
    return "Next";
  };

  const getLatLong = () => {
    const PILocation = realm.objects("PILocation");
    const latitude = PILocation[0]?.lat;
    const longitude = PILocation[0]?.long;
    if (
      locationRetryCount < 2 &&
      (latitude === undefined || longitude === undefined)
    ) {
      locationRetryCount += 1;
      Alert.alert(
        "Turn Off/On Location",
        "Oops! We are not able to fetch your location details. Please try again after turning Off and then On"
      );
      return;
    } else {
      setCurrentLatitude(latitude);
      setCurrentLongitude(longitude);
    }
  };

  // for gatting time
  const getTime = () => {
    const date = new Date().getDate();
    const month = new Date().getMonth() + 1;
    const year = new Date().getFullYear();

    //Time calulation for the display on preciew Screen
    const hour = new Date().getHours();
    const minute = new Date().getMinutes();
    const seconds = new Date().getSeconds();
    setDateTime(
      date +
        "/" +
        month +
        "/" +
        year +
        " - " +
        hour +
        ":" +
        minute +
        ":" +
        seconds
    );
  };

  const getStopVideoInstruction = (instruction) => {
    const imageName = instruction?.imageName;
    if (imageName === VDO) {
      return (
        <View style={styles.stopVideoInstructionView}>
          <TouchableOpacity
            style={{ justifyContent: "center", alignItems: "center" }}
            onPress={() => {
              if (!disableStopVideoButton) {
                stopScreenRecordingCallback(true);
                /* only able to turn off the torch at the end */
                if (props.toggleTorch !== undefined && torchState) {
                  props.toggleTorch();
                }
              }
            }}
          >
            <Icon
              name="stop-circle-o"
              size={48}
              color={disableStopVideoButton ? Colors.inactiveText : "red"}
              style={{ backgroundColor: "transparent", paddingHorizontal: 5 }}
            />
          </TouchableOpacity>
        </View>
      );
    }
  };

  const getStepVideoInstruction = (instruction) => {
    const imageName = instruction?.imageName;
    if (imageName === VDO) {
      return (
        <Steps
          navigation={props.navigation}
          nextStepCallback={nextStepRef}
          prevStepCallback={prevStepRef}
          resetCallback={resetStepRef}
          instructionArr={instructionArr}
        />
      );
    }
  };

  const instruction = (instruction) => {
    const title = instruction?.TSurveyInstructions[0]?.instructionData;
    const imageName = instruction?.imageName;
    if (imageName !== VDO) {
      return (
        <View style={styles.instructionViewMain}>
          <View style={styles.dateTimeView}>
            <Text style={styles.instructionDateTimeText}>
              {"Lat:" +
                " " +
                currentLatitude +
                "  " +
                "Lon:" +
                "  " +
                currentLongitude +
                "  " +
                "    |    " +
                "Digit App" +
                "    |    " +
                dateTime}
            </Text>
          </View>
          <View style={styles.instructionView}>
            <Text style={styles.instructionText}>{title}</Text>
            <PrimaryButton
              style={styles.instructionButton}
              disabled={false}
              onPress={() => {
                getButtonAction(instruction)();
              }}
            >
              {getButtonName(instruction)}
            </PrimaryButton>
          </View>
        </View>
      );
    }
  };

  const isInstructionWithImage = (instruction) => {
    if (instruction == undefined) {
      return false;
    }
    if (instruction.type === "input" || instruction.imageName === DBD) {
      return true;
    } else if (
      instruction.type === "image" &&
      (instruction.imageName === RCB1 || instruction.imageName === RCB2)
    ) {
      return false;
    } else {
      return false;
    }
  };

  const startVideo = async () => {
    console.log(
      "**************************** startVideo ************************************************"
    );
    if (props.startScreenRecording) {
      setDisableStopVideoButton(true);
      posePeriodIntervalFun();
      console.log("*** 0 startAnim before start");
      const res = await props.startScreenRecording();
      console.log("*** 1 startAnim after start " + JSON.stringify(res));
      if (res && animatedRef.current !== undefined) {
        animatedRef.current[0].start(() => {
          console.log("*** startAnim animation finished");
        });
        console.log("**** 2 startVideo Animation started");
        const [pauseTimer, resumeTimer] = Timer(() => {
          pauseTimerRef.current();
          console.log("*** 4 startAnim animation timeout finished");
          stopScreenRecordingCallback(false);
        }, ANIM_DURATION);
        resumeTimerRef.current = resumeTimer;
        pauseTimerRef.current = pauseTimer;
        // timeoutRef.current = setTimeout(() => {
        //   if(timeoutRef.current)
        //   clearTimeout(timeoutRef.current)
        //   console.log("*** 4 startAnim animation timeout finished");
        //   stopScreenRecordingCallback(false);
        // }, ANIM_DURATION);
      } else {
        setStartRecording(true);
        Alert.alert("Recording Cancelled!");
      }
    }
  };

  const StartRecordingMainView = () => {
    return (
      <View style={styles.startRecordingView}>
        <View style={styles.startRecordingSubView}>
          <Text style={styles.textTitle}>Please be near chassis number</Text>
          <View style={styles.lineView} />
          <PrimaryButton
            style={styles.startRecordingButton}
            onPress={() => {
              /* @VEHICLEINSTRUCTIONS 
              instructions modal pops up
              on end of modal
              startVideo will execute
              */
              setEaseAnimation(true); // Logic to just change the status,
              setStartRecording(false);
              startVideo();
            }}
          >
            {"Start Recording"}
          </PrimaryButton>
          <Text style={styles.textSubTitle}>
            Please note: once begin recording, you'll have to complete the
            entire inspection
          </Text>

          {/* adding flash support */}
          <View
            style={{
              flexDirection: "row",
              alignItems: "center",
              alignSelf: "center",
            }}
          >
            <Text style={[style.muliBoldDB, { color: Colors.white }]}>
              Flash {torchState ? "Off" : "On"}
            </Text>
            <TouchableOpacity
              style={styles.flashIconButtonSurface}
              onPress={() => {
                console.log("^^^^^ STARTED TORCH ^^^^^^");
                if (props.toggleTorch) {
                  props.toggleTorch();
                  setTorchState(!torchState);
                }
              }}
            >
              <MIcon
                name={torchState ? "flash-on" : "flash-off"}
                size={24}
                color={Colors.white}
                style={{ backgroundColor: "transparent", paddingHorizontal: 5 }}
              />
            </TouchableOpacity>
          </View>
        </View>
      </View>
    );
  };

  useEffect(() => {
    // if (instructionManager.imageName === RCB1) {
    //   setEnableRCBPreview(true);
    // } else {
    //   setEnableRCBPreview(false);
    // }
    if (instructionManager.imageName === VDO) {
      setStartRecording(true);
      setInstructionsModalVisible(true);
      setDisableStopVideoButton(true);
      // setInstructionsModalVisible(false);
    } else {
      setStartRecording(false);
      setEaseAnimation(false);
      setInstructionsModalVisible(false);
    }
  }, [instructionManager.imageName]);

  const stopIt = async () => {
    // if(animatedRef.current){
    //   clearTimeout(animatedRef.current)
    // }
    if (extraTimeRef.current) {
      clearTimeout(extraTimeRef.current);
    }
    if (posePeriodInterval.current) {
      clearInterval(posePeriodInterval.current);
      if (resetStepRef.current) {
        resetStepRef.current();
      }
    }
    if (props.stopScreenRecording) {
      console.log("*** 6 startAnim animation finished");
      await props.stopScreenRecording();
      setStartRecording(true);
    }
    setDisableStopVideoButton(true);
  };

  const stopScreenRecordingCallback = (userStopped = true) => {
    if (userStopped) {
      console.log("*** 5 stopAnim  user finished");
      stopIt();
    } else {
      setDisableStopVideoButton(false);
      extraTimeRef.current = setTimeout(() => {
        // allow extra time once ANIM_DURATION finish
        console.log("*** or 5 stopAnim extra timeout finished");
        stopIt();
      }, EXTRA_TIME);
    }
  };

  const InstructionsScreenModal = () => {
    return (
      <View>
        <Modal
          // isVisible={instructionsModalVisible}
          isVisible={true}
          headerImageVisible={false}
          shouldHeaderVisible={false}
        >
          <InstructionsScreen
            startRecording={() => {
              setInstructionsModalVisible(false);
            }}
          />
        </Modal>
      </View>
    );
  };

  var Timer = (callback, delay) => {
    var timerId,
      start,
      remaining = delay;

    const pauseTimer = () => {
      clearTimeout(timerId);
      timerId = null;
      remaining -= Date.now() - start;
      return remaining;
    };

    const resumeTimer = () => {
      if (timerId) {
        return;
      }
      start = Date.now();
      timerId = setTimeout(callback, remaining);
    };
    resumeTimer();
    return [pauseTimer, resumeTimer];
  };

  const getFlashButton = (instruction) => {
    const imageName = instruction?.imageName;
    if (imageName === RCB1 || imageName === RCB2) {
      return (
        <View style={styles.flashView}>
          <TouchableOpacity
            style={styles.flashIconButtonSurface}
            onPress={() => {
              console.log("^^^^^ STARTED TORCH ^^^^^^");
              if (props.toggleTorch !== undefined) {
                setTorchState(!torchState);
                props.toggleTorch();
              }
            }}
          >
            <MIcon
              name={torchState ? "flash-on" : "flash-off"}
              size={36}
              color={Colors.white}
              style={{ backgroundColor: "transparent", paddingHorizontal: 5 }}
            />
          </TouchableOpacity>
        </View>
      );
    }
  };

  return (
    <ViewShot style={styles.mainView} ref={viewShotRef}>
      <View style={styles.mainView}>
        {easeAnimation && (
          <AnimationEaseLinear
            animationListener={animationListener}
            onAnimationFinish={() => {}}
            ref={animatedRef}
          />
        )}
        {isInstructionWithImage(instructionManager) &&
          instructionWithImage(instructionManager)}
        {!isInstructionWithImage(instructionManager) &&
          instruction(instructionManager)}
        {/* {!isInstructionWithImage(instructionManager) &&
          enableRCBPreview &&
          goToRCBPreview(instructionManager)} */}
        {imagePreview && (
          <Image
            key={instructionManager.displayOrder}
            style={styles.previewImageView}
            resizeMode="cover"
            source={{ uri: imagePreviewUri }}
          />
        )}
        {startRecording && StartRecordingMainView()}
        {getStopVideoInstruction(instructionManager)}
        {getFlashButton(instructionManager)}
        {getStepVideoInstruction(instructionManager)}
      </View>
      {instructionsModalVisible && InstructionsScreenModal()}
    </ViewShot>
  );
});

const styles = StyleSheet.create({
  mainView: {
    flex: 1,
    height: "100%",
    zIndex: zIndex_main_view,
    width: "100%",
    position: "absolute",
    backgroundColor: "transparent",
    justifyContent: "flex-end",
  },
  previewImageView: {
    position: "absolute",
    zIndex: zIndex_snapshot_preview_image,
    width: "100%",
    alignSelf: "center",
    height: "100%",
    backgroundColor: "transparent",
  },
  instructionImageView: {
    flex: 1,
    zIndex: zIndex_image_instruction,
    marginHorizontal: "20%",
    flexDirection: "column",
    justifyContent: "center",
    backgroundColor: "white",
  },
  stencilStyle: {
    flex: 0.6,
    width: "100%",
    alignSelf: "center",
    height: "100%",
    backgroundColor: "transparent",
  },
  instructionViewMain: {
    height: "25%",
    width: "100%",
    zIndex: zIndex_bottom_instruction,
    flexDirection: "column",
    justifyContent: "center",
    backgroundColor: "transparent",
  },
  dateTimeView: {
    flex: 0.4,
    height: "100%",
    width: "100%",
    flexDirection: "row",
    justifyContent: "flex-start",
    backgroundColor: "transparent",
  },
  instructionView: {
    flex: 0.8,
    height: "100%",
    width: "100%",
    flexDirection: "row",
    justifyContent: "flex-start",
    backgroundColor: "rgba(0,0,0,0.3)",
  },
  stopVideoInstructionView: {
    position: "absolute",
    // height: "100%",
    top: 0,
    bottom: 0,
    right: 0,
    zIndex: zIndex_stop_recording,
    // width: "100%",
    flexDirection: "row",
    justifyContent: "center",
    alignItems: "center",
    backgroundColor: "transparent",
  },
  flashView: {
    position: "absolute",
    justifyContent: "center",
    alignItems: "center",
    top: 0,
    bottom: 0,
    right: 0,
    zIndex: zIndex_flash_torch,
  },
  flashViewVDO: {
    position: "absolute",
    justifyContent: "center",
    alignItems: "center",
    top: "18%",
    right: 0,
    zIndex: zIndex_flash_torchVDO,
  },
  flashIconButtonSurface: {
    justifyContent: "center",
    alignItems: "center",
    height: 48,
    width: 48,
  },
  instructionText: {
    flex: 0.8,
    color: "white",
    alignSelf: "center",
    marginLeft: 10,
    fontFamily: "Muli-Bold",
    fontSize: 18,
  },
  instructionDateTimeText: {
    color: "white",
    marginLeft: 10,
    fontFamily: "Muli-Regular",
    fontSize: 18,
  },
  instructionButton: {
    flex: 0.2,
    height: "10%",
    marginRight: 10,
    alignSelf: "center",
  },
  bottomInstructionView: {
    flex: 0.2,
    height: "15%",
    width: "100%",
    flexDirection: "row",
    justifyContent: "center",
    backgroundColor: "white",
  },
  bottmInstructionText: {
    flex: 0.7,
    color: "black",
    alignSelf: "center",
    marginLeft: 10,
    fontFamily: "Muli-Bold",
  },
  bottomInstructionButton: {
    flex: 0.3,
    height: "10%",
    marginRight: 10,
    alignSelf: "center",
  },
  topInstructionView: {
    flex: 0.2,
    width: "100%",
    height: "100%",
    justifyContent: "center",
    backgroundColor: "white",
  },
  topInstructionText: {
    color: "black",
    alignSelf: "flex-start",
    marginLeft: 10,
    fontFamily: "Muli-Bold",
  },
  startRecordingView: {
    position: "absolute",
    zIndex: zIndex_start_recording,
    width: "100%",
    height: "100%",
    justifyContent: "center",
    backgroundColor: "transparent",
  },
  startRecordingSubView: {
    width: "60%",
    height: "60%",
    alignSelf: "center",
    borderRadius: 10,
    justifyContent: "center",
    backgroundColor: "rgba(0, 0, 0, 0.8)",
  },
  textTitle: {
    color: "white",
    fontSize: 12,
    fontFamily: "Muli-Bold",
    paddingHorizontal: 10,
    alignSelf: "center",
  },
  textSubTitle: {
    color: Colors.inactiveText,
    fontSize: 10,
    fontFamily: "Muli-Regular",
    alignSelf: "center",
    paddingHorizontal: 10,
    marginVertical: 5,
  },
  lineView: {
    width: "80%",
    height: 0.5,
    marginVertical: 10,
    backgroundColor: Colors.inactiveText,
    alignSelf: "center",
  },
  startRecordingButton: {
    width: "50%",
    height: "10%",
    alignSelf: "center",
  },
});

export default {
  StencilAnimation,
  ANIM_DURATION,
  DURATION_PERIOD,
  NO_OF_POSES,
  OBJ_WIDTH,
  ANIMATION_TRACK,
};
